@@ -32,6 +32,9 @@
 #include "expressiontypechecker.h"
 #include "apply.h"
 #include "providederivative.h"
+#include "polynomial.h"
+#include "transformation.h"
+#include "substituteexpression.h"
 
 // #define SCRIPT_PROFILER
 
@@ -703,7 +706,6 @@ Object* Analyzer::operate(const Apply* c)
 			QString* error=0;
 			if(count>=2) {
 				Apply::const_iterator it = c->firstValue(), itEnd=c->constEnd();
-				
 				ret = calc(*it);
 				++it;
 				bool stop=isNull(opt, ret);
@@ -1099,16 +1101,6 @@ void Analyzer::simplify()
 	}
 }
 
-void Analyzer::levelOut(Apply *c, Apply *ob, Apply::iterator &pos)
-{
-	Apply::iterator it = ob->firstValue();
-	for(; it!=ob->end(); pos++) {
-		pos=c->m_params.insert(pos, *it);
-		
-		it=ob->m_params.erase(it);
-	}
-}
-
 template <class T, class Tit>
 void Analyzer::iterateAndSimp(T* v)
 {
@@ -1164,27 +1156,52 @@ Object* Analyzer::simp(Object* root)
 	return root;
 }
 
+Object* applyTransformations(Object* root, const QList<Transformation>& trans)
+{
+	foreach(const Transformation& t, trans) {
+		Object* o = t.applyTransformation(root);
+		if(o) {
+			delete root;
+			return o;
+		}
+	}
+	return root;
+}
+
+QList<Transformation> simplifications()
+{
+	static QList<Transformation> ret;
+	if(KDE_ISUNLIKELY(ret.isEmpty())) {
+		//divide
+		ret += Transformation(Transformation::parse("f/f"), Transformation::parse("1"));
+		ret += Transformation(Transformation::parse("f/1"), Transformation::parse("f"));
+		
+		//power
+		ret += Transformation(Transformation::parse("0**k"), Transformation::parse("0"));
+		ret += Transformation(Transformation::parse("1**k"), Transformation::parse("1"));
+		ret += Transformation(Transformation::parse("x**1"), Transformation::parse("x"));
+		ret += Transformation(Transformation::parse("(x**y)**z"), Transformation::parse("x**(y*z)"));
+		
+		//ln
+		ret += Transformation(Transformation::parse("ln e"), Transformation::parse("1"));
+	}
+	
+	return ret;
+}
+
 Object* Analyzer::simpApply(Apply* c)
 {
 	Object* root=c;
 	Apply::iterator it;
 	Operator o = c->firstOperator();
-	bool d;
 	
 	switch(o.operatorType()) {
 		case Operator::times:
 			for(it=c->firstValue(); c->countValues()>1 && it!=c->end();) {
-				d=false;
+				bool d=false;
 				*it = simp(*it);
-				if((*it)->isApply()) {
-					Apply *intr = static_cast<Apply*>(*it);
-					if(intr->firstOperator()==o) {
-						levelOut(c, intr, it);
-						d=true;
-					}
-				}
 				
-				if(!d && (*it)->type() == Object::value) {
+				if((*it)->type() == Object::value) {
 					Cn* n = (Cn*) (*it);
 					if(n->value()==1. && c->countValues()>1) { //1*exp=exp
 						d=true;
@@ -1203,188 +1220,39 @@ Object* Analyzer::simpApply(Apply* c)
 				}
 			}
 			
-			if(c->isUnary()) {
-				Apply *aux=c;
-				root=*c->firstValue();
-				*aux->firstValue()=0;
-				delete aux;
-			} else {
-				Object *ret=simpScalar(c);
-				if(ret->isApply()) {
-					c=static_cast<Apply*>(ret);
-					ret=simpPolynomials(c);
-					c=ret->isContainer() ? static_cast<Apply*>(ret) : 0;
-				} else
-					c=0;
-				
-				if(c && c->isEmpty()) {
-					delete root;
-					root = new Cn(0.);
-				} else
-					root=ret;
-			}
+			root=simpPolynomials(c);
 			break;
 		case Operator::minus:
 		case Operator::plus: {
-			bool somed=false, lastdel=false;
-			it=c->end()-1;
-			Object* first=*c->firstValue();
-			
-			bool done=false;
-			
-			for(; !done; --it) {
-				done=it==c->m_params.begin();
-				lastdel=false;
+// 			qDebug() << "........................" << c->toString();
+			for(Apply::iterator it=c->begin(), itEnd=c->end(); it!=itEnd; ++it) {
 				*it = simp(*it);
-				
-				d=false;
-				
-				if((*it)->isApply()) {
-					Apply *intr = (Apply*) *it;
-					Operator op=intr->firstOperator();
-					if(op==o || (*it!=first && op==Operator::plus && o==Operator::minus))
-					{
-						if(!(intr->isUnary() && op==Operator::minus))
-						{
-							levelOut(c, intr, it);
-							d=true;
-						}
-					}
-				}
-				
-				if(!d && ((*it)->type()==Object::value || ((*it)->type()==Object::vector && !hasVars(*it))) && (*it)->isZero()) {
-					d=true;
-					lastdel=true;
-				}
-				
-				if(d) {
-					somed=true;
-					delete *it;
-					if(first==*it) first=0;
-					it = c->m_params.erase(it);
-				}
 			}
 			
-// 			qDebug()<< "KOKOKO" << delme << c->toString() << lastdel;
-			
-			if(lastdel && o==Operator::minus && c->countValues()>1) {
-				Apply::iterator it=c->firstValue();
-				Apply* cc=new Apply;
-				cc->appendBranch(new Operator(Operator::minus));
-				cc->appendBranch(*it);
-				*it=cc;
-			}
-			root=c;
-			
-// 			qDebug()<< "PEPEPE" << delme << c->toString();
+// 			qDebug()<< "PEPEPE" << c->toString();
 			if(c->isUnary()) {
-				if(o==Operator::plus || (somed && !lastdel)) {
-					root=*c->firstValue();
+				if(o==Operator::minus && (*c->firstValue())->type()==Object::value) {
+					Cn* v = static_cast<Cn*>(*c->firstValue());
+					v->rvalue() *= -1;
+					
+					root=v;
 					*c->firstValue()=0;
 					delete c;
 					c=0;
-				} else if((*c->firstValue())->isApply()) {
-					Apply *c1=(Apply*) *c->firstValue();
-					if( c1->firstOperator()==Operator::minus &&
-						c1->isUnary())
-					{
-						root=*c1->firstValue();
-						*c->firstValue()=0;
-						*c1->firstValue()=0;
-						delete c;
-						delete c1;
-						c=0;
-					}
 				}
 			} else {
-				root=simpScalar(c);
-				
-				if(root->isApply()) {
-					c=static_cast<Apply*>(root);
-					
-					root=simpPolynomials(c);
-					
-					c=root->isApply() ? static_cast<Apply*>(root) : 0;
-				} else
-					c=0;
+				root=simpPolynomials(c);
+				c=0;
 			}
 // 			qDebug()<< "PAAPPA" << root->toString();
-			
-			if(c && c->isEmpty()) {
-				delete root;
-				root = new Cn(0.);
+			static QList<Transformation> addTrans;
+			if(addTrans.isEmpty()) {
+				addTrans += Transformation(Transformation::parse("--x"), Transformation::parse("x"));
+				addTrans += Transformation(Transformation::parse("-a--b"), Transformation::parse("b-a"));
 			}
+			
+			root = applyTransformations(root, addTrans);
 		}	break;
-		case Operator::power: {
-			for(it = c->firstValue(); it!=c->end(); ++it)
-				*it = simp(*it);
-			
-			if(c->m_params[1]->type()==Object::value) {
-				Cn *n = (Cn*) c->m_params[1];
-				if(n->value()==0.) { //0*exp=0
-					delete root;
-					root = new Cn(1.);
-					break;
-				} else if(n->value()==1.) { 
-					root = c->m_params[0];
-					c->m_params[0]=0;
-					delete c;
-					break;
-				}
-			}
-			
-			if(c->m_params[0]->isApply()) {
-				Apply *cp = (Apply*) c->m_params[0];
-				if(cp->firstOperator()==Operator::power) {
-					c->m_params[0] = cp->m_params[0];
-					
-					Apply *cm = new Apply;
-					cm->appendBranch(new Operator(Operator::times));
-					cm->appendBranch(c->m_params[1]);
-					cm->appendBranch(cp->m_params[1]);
-					c->m_params[1] = cm;
-					
-					cp->m_params[0]=0;
-					cp->m_params[1]=0;
-					delete cp;
-					c->m_params[1]=simp(c->m_params[1]);
-				}
-			}
-		} break;
-		case Operator::divide:
-			for(it = c->firstValue(); it!=c->end(); ++it)
-				*it = simp(*it);
-			
-			Object *f, *g; //f/g
-			f=*c->firstValue();
-			g=*(c->firstValue()+1);
-			
-			if(equalTree(f, g)) {
-				delete root;
-				root = new Cn(1.);
-			} else if(g->type()==Object::value) {
-				Cn* gnum=(Cn*) g;
-				if(gnum->value()==1.) {
-					*c->firstValue()=0;
-					delete root;
-					root=f;
-					break;
-				}
-			}
-			break;
-		case Operator::ln:
-			it = c->firstValue();
-			if((*it)->type()==Object::variable) {
-				Ci *val = (Ci*) *it;
-				if(val->name()=="e") {
-					delete root;
-					root = new Cn(1.);
-					break;
-				}
-			} else {
-				*it = simp(*it);
-			}
-			break;
 		//TODO: extend to ::product
 		case Operator::sum: {
 			
@@ -1495,16 +1363,66 @@ Object* Analyzer::simpApply(Apply* c)
 			
 		}	break;
 		case Operator::eq: {
-			Apply::iterator it=c->firstValue()+1, itEnd=c->end();
+			bool alleq=true, existsjustvar=false, allButFirstZero=false;
+			QStringList deps = AnalitzaUtils::dependencies(c, QStringList());
 			
-			bool alleq=true;
-			for(; alleq && it!=itEnd; ++it) {
-				alleq=equalTree(*c->firstValue(), *it);
+			for(Apply::iterator it=c->firstValue(), itEnd=c->end(); it!=itEnd; ++it) {
+				*it = simp(*it);
+				alleq = alleq && equalTree(*c->firstValue(), *it);
+				existsjustvar = existsjustvar || (*it)->type()==Object::variable;
+				allButFirstZero = (it==c->firstValue() || (*it)->isZero());
 			}
 			
 			if(alleq) {
 				delete c;
 				root = new Cn(true);
+			} else if(!existsjustvar && deps.size()==1) {
+				if(allButFirstZero)
+					root = *c->firstValue();
+				else {
+					Apply* a = new Apply;
+					a->appendBranch(new Operator(Operator::minus));
+					
+					for(Apply::const_iterator it=c->constBegin(), itEnd=c->constEnd(); it!=itEnd; ++it) {
+						a->appendBranch(*it);
+					}
+					c->m_params.clear();
+					delete c;
+					
+					root = simp(a);
+				}
+				
+				if(root->type()==Object::apply) {
+					QList<Object*> r = findRoots(deps.first(), static_cast<Apply*>(root));
+					
+					if(r.isEmpty()) {
+						Apply* na = new Apply;
+						na->appendBranch(new Operator(Operator::eq));
+						na->appendBranch(root);
+						na->appendBranch(new Cn(0.));
+						root=na;
+					} else if(r.size() == 1) {
+						Apply* a = new Apply;
+						a->appendBranch(new Operator(Operator::eq));
+						a->appendBranch(new Ci(deps.first()));
+						a->appendBranch(r.first());
+						root = a;
+					} else {
+						Apply* na = new Apply;
+						na->appendBranch(new Operator(Operator::_or));
+						foreach(Object* o, r) {
+							Apply* a = new Apply;
+							a->appendBranch(new Operator(Operator::eq));
+							a->appendBranch(new Ci(deps.first()));
+							a->appendBranch(o);
+							na->appendBranch(a);
+						}
+						root = na;
+					}
+				} else if(!root->isZero()) {
+					delete root;
+					root = new Cn(false);
+				}
 			}
 			
 		}	break;
@@ -1547,257 +1465,154 @@ Object* Analyzer::simpApply(Apply* c)
 				c->domain()=simp(c->domain());
 			
 			iterateAndSimp<Apply, Apply::iterator>(c);
+			
+			root = applyTransformations(root, simplifications());
 			break;
 	}
 	
 	return root;
 }
 
-Object* Analyzer::simpScalar(Apply * c)
+QList<Object*> Analyzer::findRoots(const QString& dep, const Object* o)
 {
-	Object *value=0;
-	Apply::iterator i = c->firstValue();
-	Operator o = c->firstOperator();
-	bool firstvalue = i!=c->end() && ((*i)->type()==Object::value || ((*i)->type()==Object::vector && !hasVars(*i)));
-	for(; i!=c->end();) {
-		bool d=false;
-		
-		//TODO: hasVars needed? should have already been simplifyed before, just check type==cn
-		if((*i)->type()==Object::value || ((*i)->type()==Object::vector && !hasVars(*i))) {
-			Object* aux = *i;
+	switch(o->type()) {
+		case Object::apply:		return findRootsApply(dep, static_cast<const Apply*>(o));
+		case Object::variable:	return QList<Object*>() << new Cn(0.);
+		default:
+			return QList<Object*>();
+	}
+}
+
+QList<Object*> Analyzer::findRootsApply(const QString& dep, const Apply* a)
+{
+	Operator op=a->firstOperator();
+	QList<Object*> ret;
+	switch(op.operatorType()) {
+		case Operator::plus:
+		case Operator::minus: {
+			Object* varTree = 0;
+			//f(x)-w=0 => f(x)=w => x=f-1(x)
+			for(Apply::const_iterator it=a->constBegin(), itEnd=a->constEnd(); it!=itEnd; ++it) {
+				bool hasvars = hasVars(*it);
+				if(hasvars && varTree) {
+					varTree = 0; //we don't support having more than 1 variable in the minus yet
+					return QList<Object*>();
+				}
+				
+				if(hasvars && ((*it)->type() == Object::variable || (*it)->isApply())) {
+					if((*it)->isApply()) {
+						const Apply* a = static_cast<const Apply*>(*it);
+						if(!a->isUnary() || a->firstOperator().inverse().operatorType()==Operator::none)
+							break;
+					}
+					varTree = *it;
+				}
+			}
 			
-			if(value) {
-				QString* err=0;
-				value=Operations::reduce(o.operatorType(), value, aux, &err);
-				delete err;
-			} else
-				value=aux;
-			d=true;
-		}
-		if(d)
-			i = c->m_params.erase(i);
-		else
-			++i;
-	}
-	
-	if(value) {
-		if(value->isZero())
-			delete value;
-		else if(o==Operator::plus || (!firstvalue && o==Operator::minus))
-			c->appendBranch(value);
-		else
-			c->insertBranch(c->firstValue(), value);
-	}
-	return c;
-}
-
-namespace Analitza
-{
-typedef QPair<double, Object*> Monomial;
-
-Object* createMono(const Operator& o, const Monomial& p)
-{
-	Operator::OperatorType mult = o.multiplicityOperator();
-	
-	Object* toAdd=0;
-	if(p.first==0.) {
-		delete p.second;
-	} else if(p.first==1.) {
-		toAdd=p.second;
-	} else if(p.first==-1. && mult==Operator::times) {
-		Apply *cint = new Apply;
-		cint->appendBranch(new Operator(Operator::minus));
-		cint->appendBranch(p.second);
-		toAdd=cint;
-	} else if(mult==Operator::times && p.second->isApply() && static_cast<Apply*>(p.second)->firstOperator()==Operator::times) {
-		Apply* res = static_cast<Apply*>(p.second);
-		res->prependBranch(new Cn(p.first));
-		toAdd=res;
-	} else {
-		Apply *cint = new Apply;
-		cint->appendBranch(new Operator(mult));
-		if(mult==Operator::times) {
-			cint->appendBranch(new Cn(p.first));
-			cint->appendBranch(p.second);
-		} else {
-			cint->appendBranch(p.second);
-			cint->appendBranch(new Cn(p.first));
-		}
-		toAdd=cint;
-	}
-	return toAdd;
-}
-
-Monomial constructMonomial(const Operator& o, Object* o2, bool& sign)
-{
-	bool ismono=false;
-	Monomial imono;
-	Operator::OperatorType mult = o.multiplicityOperator();
-	
-	if(o2->isApply()) {
-		Apply *cx = (Apply*) o2;
-		if(cx->firstOperator()==mult) {
-			if(cx->countValues()==2) {
-				bool valid=false;
-				int scalar, var;
-				
-				if(mult!=Operator::power && cx->m_params[0]->type()==Object::value) {
-					scalar=0;
-					var=1;
-					valid=true;
-				} else if(cx->m_params[1]->type()==Object::value) {
-					scalar=1;
-					var=0;
-					valid=true;
+			if(varTree) {
+				Apply* na = 0;
+				Object* value = 0;
+				if(a->countValues()>2) {
+					na = new Apply;
+					na->appendBranch(new Operator(a->firstOperator().inverse()));
+					value = na;
 				}
 				
-				if(valid) {
-					Cn* sc= (Cn*) cx->m_params[scalar];
-					imono.first = sc->value();
-					imono.second = cx->m_params[var];
-					
-					cx->m_params[var]=0;
-					delete cx;
-					
-					ismono=true;
-				}
-			} else if(mult==Operator::times) {
-				imono.first=1;
-				Apply::iterator it=cx->firstValue(), itEnd=cx->end();
-				QVector<Object*> vars;
-				QVector<Object*> values;
-				
-				for(; it!=itEnd; ++it) {
-					if((*it)->type()==Object::value) {
-						imono.first *= static_cast<Cn*>(*it)->value();
-						values += *it;
-						ismono=true;
-					} else {
-						vars += *it;
+				for(Apply::const_iterator it=a->constBegin(), itEnd=a->constEnd(); it!=itEnd; ++it) {
+					if(*it != varTree) {
+						if(na)
+							na->appendBranch((*it)->copy());
+						else {
+							Q_ASSERT(!value);
+							value = (*it)->copy();
+						}
 					}
 				}
 				
-				if(ismono) {
-					cx->m_params = vars;
-					imono.second = cx;
-					qDeleteAll(values);
+				Q_ASSERT(value);
+				if(varTree->isApply()) {
+					Operator inv = static_cast<const Apply*>(varTree)->firstOperator().inverse();
+					Apply* aa = new Apply;
+					aa->appendBranch(inv.copy());
+					aa->appendBranch(value);
+					value = calc(aa);
+					delete aa;
+				}
+				
+				if(op==Operator::minus) {
+					ret += value;
+				} else {
+					Apply* aa = new Apply;
+					aa->appendBranch(new Operator(Operator::minus));
+					aa->appendBranch(value);
+					ret += calc(aa);
+					delete aa;
 				}
 			}
-		} else if(cx->firstOperator()==Operator::minus && cx->isUnary()) {
-			imono = constructMonomial(o, *cx->firstValue(), sign);
-			*cx->firstValue()=0;
-			delete cx;
-			ismono=true;
+		}	break;
+		case Operator::times:
+			for(Apply::const_iterator it=a->constBegin(), itEnd=a->constEnd(); it!=itEnd; ++it) {
+				ret += findRoots(dep, static_cast<Apply*>(*it));
+			}
+			break;
+		case Operator::divide: { // f/g
+			Object* f = *a->firstValue();
+			Object* g = *(a->firstValue()+1);
+			ret = findRoots(dep, f);
+			
+			for(QList<Object*>::iterator it = ret.begin(), itEnd=ret.end(); it!=itEnd; ) {
+				bool erase = false;
 				
-			if(o==Operator::times)
-				sign = !sign;
-			else if(o==Operator::plus || o==Operator::minus)
-				imono.first *= -1;
-		}
+				Object* val = testResult(g, dep, *it);
+				erase = val->isZero();
+				delete val;
+				
+				if(erase) {
+					delete *it;
+					it = ret.erase(it);
+				} else
+					++it;
+			}
+		}	break;
+		default: {
+			Operator inv = op.inverse();
+			if(inv.operatorType()!=Operator::none && a->isUnary()) {
+				Apply* aa = new Apply;
+				aa->appendBranch(inv.copy());
+				aa->appendBranch(new Cn(0.));
+				ret += calc(aa);
+				delete aa;
+			}
+		}	break;
 	}
-	
-	if(!ismono) {
-		imono.first = 1.;
-		imono.second = o2;
-	}
-	
-	return imono;
+	return ret;
 }
 
+Object* Analyzer::testResult(const Object* o, const QString& var, const Object* val)
+{
+	SubstituteExpression s;
+	QMap<QString, const Object*> subs;
+	subs.insert(var, val);
+	
+	QScopedPointer<Object> sometree(s.run(o, subs));
+	return calc(sometree.data());
 }
 
 Object* Analyzer::simpPolynomials(Apply* c)
 {
-	Q_ASSERT(c!=0 && dynamic_cast<Apply*>(c));
+	Q_ASSERT(c!=0 && c->isApply());
 	
-	QList<Monomial> monos;
-	Operator o(c->firstOperator());
-	bool sign=true, first=true;
+	Polynomial monos(c);
 	
-	for(Apply::const_iterator it=c->constBegin(), itEnd=c->constEnd(); it!=itEnd; ++it, first=false) {
-		Monomial imono = constructMonomial(o, *it, sign);
-		
-		if(o==Operator::minus && !first)
-			imono.first*=-1;
-		
-		bool found = false;
-		QList<Monomial>::iterator it1(monos.begin());
-		for(; it1!=monos.end(); ++it1) {
-			Object *o1=it1->second, *o2=imono.second;
-			found = equalTree(o1, o2);
-			if(found)
-				break;
-		}
-		
-// 		qDebug() << "->" << c->toString() << c->firstOperator().toString() << found;
-		if(found) {
-			it1->first += imono.first;
-			delete imono.second;
-			
-			if(it1->first==0.) {
-				delete it1->second;
-				monos.erase(it1);
-			}
-		} else {
-			monos.append(imono);
-		}
-	}
 	c->m_params.clear();
-	
 	delete c;
 	c=0;
 	
-	Object *root=0;
-	if(monos.count()==1) {
-		root=createMono(o, monos.first());
-	} else if(monos.count()>1) {
-		Apply* c= new Apply;
-		c->appendBranch(new Operator(o));
-		
-		QList<Monomial>::iterator i=monos.begin();
-		bool first=true;
-		for(; i!=monos.end(); ++i) {
-			if(!first && o==Operator::minus)
-				i->first *= -1;
-			Object* toAdd=createMono(o, *i);
-			
-			if(toAdd)
-				c->appendBranch(toAdd);
-			
-			first=false;
-		}
-		root=c;
-	}
-	
-	if(!sign && root) {
-		Apply *cn=new Apply;
-		cn->appendBranch(new Operator(Operator::minus));
-		cn->appendBranch(root);
-		root=cn;
-	} else if(!root) {
-		root=new Cn(0.);
-	}
+	Object *root=monos.toObject();
 	
 	return root;
 }
 
-int removeAll(QVector<Object*>& v, Object* o)
-{
-// 	int removed=0;
-// 	for(QVector<Object*>::iterator it=v.begin(), itEnd=v.end(); it!=itEnd;) {
-// 		if(*it==o)
-// 			it=v.erase(it);
-// 		else
-// 			++it;
-// 	}
-// 	
-// 	return removed;
-
-//TODO: remove this... -.-
-	QList<Object*> hack=v.toList();
-	return hack.removeAll(o);
-}
-
 Object* Analyzer::simpSum(Apply* c)
 {
 	Object* ret=c;
@@ -1807,15 +1622,16 @@ Object* Analyzer::simpSum(Apply* c)
 		QSet<QString> bvars=c->bvarStrings().toSet();
 		QVector<Object*> sum, out;
 		Apply::iterator it=cval->firstValue(), itEnd=cval->end();
+		int removed=0;
 		for(; it!=itEnd; ++it) {
 			if(hasTheVar(bvars, *it)) {
 				sum.append(*it);
 			} else {
 				out.append(*it);
 				*it=0;
+				++removed;
 			}
 		}
-		int removed=removeAll(cval->m_params, 0);
 		
 		if(removed>0) {
 			Apply* nc=new Apply;
@@ -1951,8 +1767,8 @@ Expression Analyzer::dependenciesToLambda() const
 		
 		Container* math=new Container(Container::math);
 		math->appendBranch(cc);
-		Expression::computeDepth(math);
 		
+		Expression::computeDepth(math);
 		return Expression(math);
 	} else {
 		return m_exp;
@@ -2039,7 +1855,7 @@ Object* Analyzer::applyAlpha(Object* o, int min)
 		switch(o->type()) {
 			case Object::container:	alphaConversion(static_cast<Container*>(o), min); break;
 			case Object::vector:	alphaConversion<Vector, Vector::iterator>(static_cast<Vector*>(o), min); break;
-			case Object::list:	alphaConversion<List, List::iterator>(static_cast<List*>(o), min); break;
+			case Object::list:		alphaConversion<List, List::iterator>(static_cast<List*>(o), min); break;
 			case Object::apply:		alphaConversion(static_cast<Apply*>(o), min); break;
 			case Object::variable: {
 				Ci *var = static_cast<Ci*>(o);
